<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Services; 

use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\CMP\Util\CmpUtilities;

class Cmp07600AlteracaoAprovacaoAfService {

    protected $connection;
    protected $smtpUtil;
    protected $entityManager;
    protected $cmpUtilities;
    protected $newCode;
    protected $environment;
    
    public function __construct ($entityManager, $smtpUtil, CmpUtilities $cmpUtilities, $newCode, $environment) {
        $this->smtpUtil = $smtpUtil; 
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->cmpUtilities = $cmpUtilities;
        $this->newCode = $newCode;
        $this->environment = $environment;
    }
    
    public function procedureGeraNovoCodigo($tabela) {
        try {
            return $this->newCode->getCodeByTableName($tabela, $this->environment->getNrOrg());
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function parseFloat($floatVar) {
        if(is_string($floatVar)) {
            $floatVar = str_replace(".", "", $floatVar);    
            $floatVar = str_replace(",", ".", $floatVar);
        }
        return $floatVar;
    }
    
    public function getSystemParamGeral(){
        
        $select = "SELECT IDSISDOS FROM PARAMGERAL";

        try
        {
            $std = $this->connection->prepare($select);
            $std->execute();
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getBranchOperator($selectParams){
        
        $select = "SELECT DISTINCT CDOPERADOR
                      FROM GRUPFILIOPER, FILGRUPFILI
                     WHERE GRUPFILIOPER.CDGRUPFILI = FILGRUPFILI.CDGRUPFILI 
                       AND FILGRUPFILI.CDFILIAL    = :P_CDFILDEST";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getTradingCenter($selectParams){
        
        $select = "SELECT NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) AS CDFILCENTCMP
                      FROM FILIAL FIL, FILIAL
                     WHERE FILIAL.CDFILIAL = :P_CDFILDEST
                       AND NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = FIL.CDFILIAL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getPermissions($selectParams){
        
        $select = "SELECT CDSUBGRUPPROD, CDGRUPPROD, IDBLOQFORNCOTAF, IDCARPRECLAF
                      FROM PARASUPRI
                     WHERE CDFILIAL = :P_CDFILCENTCMP
                       AND CDSUBGRUPPROD IS NOT NULL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getIntegration($row, $selectParams){
        
        $select = self::getIntegrationSelectByCurrentSystem($row);
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getIntegrationSelectByCurrentSystem($row){
        
        if($row["IDSISDOS"] == 3){
            
            $select = "SELECT AUTOFORNE.CDFILDEST
                        FROM AUTOFORNE, FORNECEDOR
                       WHERE AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                        AND AUTOFORNE.CDFILDEST  = :P_CDFILDEST
                        AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN
                        AND NVL(FORNECEDOR.USR_INTEGRAMESAP, 'N') = 'S'
                        AND USR_IDINTEGRAMESAP IS NOT NULL";   
            
        } else {
            
            $select = "SELECT AUTOFORNE.CDFILDEST
                          FROM AUTOFORNE
                         WHERE AUTOFORNE.CDFILDEST  = :P_CDFILDEST
                           AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN";
            
        }
        
        return $select;
        
    }
    
    public function getFrequency($selectParams){
        
        $select = "SELECT PERIENTR.CDPERIENTR, PERIENTR.NMPERIENTR
                      FROM REFORFIL, PERIENTR
                     WHERE REFORFIL.CDPERIENTR = PERIENTR.CDPERIENTR
                       AND REFORFIL.CDFORNECED = :P_CDFORNECED
                       AND REFORFIL.CDFILIAL   = :P_CDFILDEST";
                       
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function executeProcedureCarregaPeriforn($row) {
       
        $procedure = new StoredProcedure($this->connection, "CARREGA_PERIFORN");
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row["CDFILDEST"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $row["CDFORNECED"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTPROGENTAF', Param::PARAM_INPUT, $row["DTNOVAENTRAF"], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NEW_DTPROGENTAF', Param::PARAM_OUTPUT, null, null, 20));
        
        $newDate = $procedure->execute();
        return $newDate["P_NEW_DTPROGENTAF"];
        
    }
    
    public function sendEmailToProductResponsibleOperators($row) {
        $attrImg = 'width="200px"';
	    $srcLogo = $this->cmpUtilities->returnLogo('PORTAL E-COMMERCE');
        $this->cmpUtilities->getLogoAttributes($srcLogo, $attrImg);
        $lgpd = $this->cmpUtilities->returnLgpd($row['FILIAL_LOGADA']);
        
        $subject = 'Aviso de Alteração de AF';
        $assunto = "<p class='bold' style='text-align: center; color: #ED2424'>Aviso de Alteração de AF</p>";
        
        foreach ($row["OPERADORES_RESPONSAVEIS"] as $operator) {
            try {
                $mensagem = "";
                $mensagem .= "<p>" . $operator["NMOPERADOR"] . ", </p>";
                $mensagem .= "<p> Abaixo seguem as informações da autorização do fornecimento cuja alteração deve ser analisada por você. São elas: </p>";
                $mensagem .= "<ul>
                                <li><strong>AF: </strong>" . $row["NRAUTOFORN"] . "</li>
                                <li><strong>Unidade Lançamento: </strong>" . $row["CDFILLANC"] . "</li>
                                <li><strong>Unidade Destino: </strong>" . $row["NMFILDEST"] . "</li>
                                <li><strong>Data de Entrega: </strong>" . $row["DTPROGENTAF"] . "</li>
                                <li><strong>Comprador: </strong>" . $row["NMOPERADOR"] . "</li>
                                <li><strong>Fornecedor: </strong>" . $row["NMRAZSOCFORN"] . "</li>
                              </ul>";
                              
                $messageSendMail = $this->cmpUtilities->returnMessageSendMail($srcLogo, $assunto, $mensagem, $attrImg, $lgpd);
                $this->cmpUtilities->enviaEmail($row['EMAIL_LOGADO'], $subject, $operator["DSEMAILOPER"], $messageSendMail, $operator["DSEMAILOPER"]);
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    public function getAfResponsibleOperators($selectParams){
        
        $select = "SELECT DISTINCT OPERADOR.CDOPERADOR, OPERADOR.DSEMAILOPER, OPERADOR.CDSENEMAILOP, OPERADOR.NMOPERADOR
                      FROM GRUPFILIOPER, FILGRUPFILI, OPERADOR
                     WHERE GRUPFILIOPER.CDGRUPFILI = FILGRUPFILI.CDGRUPFILI 
                       AND FILGRUPFILI.CDFILIAL    = :P_CDFILDEST
                       AND GRUPFILIOPER.CDOPERADOR = OPERADOR.CDOPERADOR";
       
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function updateAllAfProductDeliveryDate($updateParams){
        
        $update = "UPDATE ITEMAUTF
                    SET DTNOVAENTRAF = :P_DTNOVAENTRAF
                   WHERE CDFILDEST = :P_CDFILDEST
                   AND CDFILLANC = :P_CDFILLANC
                   AND NRAUTOFORN = :P_NRAUTOFORN";
        
        try
        {
            $std = $this->connection->prepare($update);
            $std->execute($updateParams);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getProductsWithNewDeliveryDate($selectParams){
        
        $select = "SELECT QTNOVAPROGAF
                    FROM ITEMAUTF    
                   WHERE CDFILDEST = :P_CDFILDEST
                   AND CDFILLANC = :P_CDFILLANC
                   AND NRAUTOFORN = :P_NRAUTOFORN
                   AND QTNOVAPROGAF IS NOT NULL";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getParcmpsup($selectParams){
        
        $select = "SELECT IDOBLIBAFMAN
                      FROM PARCMPSUP
                     WHERE CDFILIAL = :CDFILIAL
                       AND NRORG = :NRORG";
        
        try
        {
            $std = $this->connection->prepare($select);
            $std->execute($selectParams);
            return $std->fetchAll();
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function getParamSupplier($params){
        
        $query = "SELECT
                     AUTOFORNE.CDFILDEST,
                     NVL(FORNECEDOR.USR_INTEGRAMESAP,'N') USR_INTEGRAMESAP,
                     FORNECEDOR.IDPERALTAFCONF,
                     NVL(AUTOFORNE.IDSITUATENDIMENTOAF,'X') IDSITUATENDIMENTOAF
                 FROM
                     AUTOFORNE,
                     FORNECEDOR
                 WHERE
                     AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                     AND AUTOFORNE.CDFILDEST =:P_CDFILDEST
                     AND AUTOFORNE.NRAUTOFORN =:P_NRAUTOFORN";

        $std = $this->connection->prepare($query);
        $std->execute($params);
        return $std->fetchAll()[0];
        
    }
    
    public function updateAf($updateParams){
        
          $update = "UPDATE AUTOFORNE
                    SET IDAFFORALIM = :IDAFFORALIM,
                        IDSITUATUAAF = :IDSITUATUAAF "
                    . $updateParams['CDGERENTE'] .    
                   "WHERE CDFILDEST = :CDFILDEST
                   AND CDFILLANC = :CDFILLANC
                   AND NRAUTOFORN = :NRAUTOFORN
                   AND NRORG = :NRORG";
        
        $params = array(
            
            "IDAFFORALIM"   => $updateParams["IDAFFORALIM"],
            "IDSITUATUAAF"  => $updateParams["IDSITUATUAAF"],
            "CDFILDEST"     => $updateParams["CDFILDEST"],
            "CDFILLANC"     => $updateParams["CDFILLANC"],
            "NRAUTOFORN"    => $updateParams["NRAUTOFORN"],
            "NRORG"         => $updateParams["NRORG"]
            
        );
        
        try
        {
            $std = $this->connection->prepare($update);
            $std->execute($params);
        }
        catch(\Exception $e)
        {
            throw $e;
        };
        
    }
    
    public function adqVerifCtPag($row) {
        $param = array(
            'P_NRAUTOFORN'  => $row['NRAUTOFORN'],
            'P_CDFILDEST'   => $row['CDFILDEST'],
            'P_CDFILLANC'   => $row['CDFILLANC']
        );
        
        $query = "SELECT COUNT( DISTINCT PROD.CDTPCTPAGAR) CONTA
                    FROM PRODUTO PROD, ITEMAUTF,
                         (SELECT P.CDPRODUTO, P.CDARVPROD,
                                 (SELECT NVL(MAX(PRODUTO.CDARVPROD), 'W')
                                    FROM PRODUTO
                                   WHERE PRODUTO.CDTPCTPAGAR IS NOT NULL
                                     AND PRODUTO.CDARVPROD = SUBSTR(P.CDARVPROD, 1, LENGTH(PRODUTO.CDARVPROD))) PRODTPCTPAGAR
                            FROM ITEMAUTF, PRODUTO P
                           WHERE ITEMAUTF.CDPRODUTO  = P.CDPRODUTO
                             AND ITEMAUTF.CDFILDEST  = :P_CDFILDEST
                             AND ITEMAUTF.CDFILLANC  = :P_CDFILLANC
                             AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN) I
                   WHERE I.PRODTPCTPAGAR     = PROD.CDARVPROD
                     AND ITEMAUTF.CDPRODUTO  = I.CDPRODUTO
                     AND ITEMAUTF.CDFILDEST  = :P_CDFILDEST
                     AND ITEMAUTF.CDFILLANC  = :P_CDFILLANC
                     AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN";
                   
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    
    public function updateCdtpctpagar($updateParams) {
        $query = "UPDATE AUTOFORNE
            SET CDTPCTPAGAR =
              (SELECT PRODUTO.CDTPCTPAGAR
              FROM PRODUTO,
                (SELECT MAX(PRODUTO.CDARVPROD) AS CDARVPROD
                FROM PRODUTO,
                  (SELECT MIN(A.CDARVPROD) AS CDARVPROD
                  FROM
                    (SELECT CDARVPROD
                    FROM ITEMAUTF,
                      PRODUTO P
                    WHERE P.CDPRODUTO       = ITEMAUTF.CDPRODUTO
                    AND ITEMAUTF.CDFILDEST  = :CDFILDEST
                    AND ITEMAUTF.CDFILLANC  = :CDFILLANC
                    AND ITEMAUTF.NRAUTOFORN = :NRAUTOFORN
                    AND ITEMAUTF.NRORG = :NRORG
                    GROUP BY CDARVPROD
                    HAVING MAX(QTPROGENTAF   *VRPREUNIAF) =
                      (SELECT MAX(QTPROGENTAF*VRPREUNIAF)
                      FROM ITEMAUTF
                      WHERE ITEMAUTF.CDFILDEST = :CDFILDEST
                      AND ITEMAUTF.CDFILLANC   = :CDFILLANC
                      AND ITEMAUTF.NRAUTOFORN  = :NRAUTOFORN
                      AND ITEMAUTF.NRORG = :NRORG
                      )
                    )A
                  ) ITEMAUTF
                WHERE ITEMAUTF.CDARVPROD LIKE PRODUTO.CDARVPROD
                  || '%'
                AND PRODUTO.CDTPCTPAGAR IS NOT NULL
                AND PRODUTO.NRORG = :NRORG
                ) TIPOCTPAGAR
              WHERE PRODUTO.CDARVPROD  = TIPOCTPAGAR.CDARVPROD
              AND PRODUTO.CDTPCTPAGAR IS NOT NULL
              AND PRODUTO.NRORG = :NRORG
              )
            WHERE AUTOFORNE.CDFILDEST  = :CDFILDEST
            AND AUTOFORNE.CDFILLANC    = :CDFILLANC
            AND AUTOFORNE.NRAUTOFORN   = :NRAUTOFORN
            AND AUTOFORNE.NRORG = :NRORG
            AND AUTOFORNE.CDTPCTPAGAR IS NULL";
            
        $params = array(
            "CDFILDEST"  => $updateParams["CDFILDEST"],
            "CDFILLANC"  => $updateParams["CDFILLANC"],
            "NRAUTOFORN" => $updateParams["NRAUTOFORN"],
            "NRORG"      => $updateParams["NRORG"]
        );
        $result = $this->entityManager->getConnection()->executeUpdate($query, $params);
        return $result;
    }
        
    public function CUSTO_AF($procParams) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'CUSTO_AF');
        $procedure->addParam(new Param('P_CDFILDEST',   Param::PARAM_INPUT, $procParams["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',   Param::PARAM_INPUT, $procParams["CDFILLANC"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN',  Param::PARAM_INPUT, $procParams["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQAPURAF', Param::PARAM_INPUT, $procParams["NRSEQAPURAF"], Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function IMPOSTO_ITEMAF($row) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'IMPOSTO_ITEMAF');
        $procedure->addParam(new Param('P_EMPRESA',    Param::PARAM_INPUT, $row["CDEMPRESA"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA',   Param::PARAM_INPUT, $row["CDINSCESTA"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FORNECEDOR', Param::PARAM_INPUT, $row["CDFORNECED"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILDEST',    Param::PARAM_INPUT, $row["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILLANC',    Param::PARAM_INPUT, $row["CDFILLANC"],   Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_LANCTOAF',   Param::PARAM_INPUT, $row["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID',   Param::PARAM_INPUT, $row["DTPROGENTAF"], Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function adqEntrItmSol($row) {
        $param = array();
        $param["P_CDFILDEST"]       = $row["CDFILDEST"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_NRAUTOFORN"]      = $row["NRAUTOFORN"];
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_DTPROGENTAF"]     = substr($row["DTPROGENTAF"], 0, 10);
        
        $query = "SELECT
                      MIN(DTENTRITMSOL) DTENTRITMSOL
                    FROM
                      ITEMAUTF
                    WHERE
                      (CDFILDEST  = :P_CDFILDEST)   AND
                      (CDFILLANC  = :P_CDFILLANC)   AND
                      (NRAUTOFORN = :P_NRAUTOFORN)  AND
                      (CDPRODUTO  = :P_CDPRODUTO)   AND
                      (CDMARCA    = :P_CDMARCA)     AND
                      (DTPROGENTAF= :P_DTPROGENTAF)";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqItemSoli($row, $adqEntrItmSol) {
        $param = array();
        $param["P_CDFILSOLI"]   = $row["CDFILDEST"];
        $param["P_DTPROGENTR"]  = count($adqEntrItmSol) > 0 ? substr($adqEntrItmSol[0]['DTENTRITMSOL'], 0, 10) : substr($row['DTPROGENTAF'], 0 , 10);
        $param["P_CDPRODUTO"]   = $row["NRAUTOFORN"];
        
        $query = "SELECT
                      ITEMSOLI.CDFILSOLI,
                      ITEMSOLI.CDFILLAN,
                      ITEMSOLI.NRSOLICMP,
                      ITEMSOLI.CDPRODUTO,
                      ITEMSOLI.DTPROGENTR,
                      ITEMSOLI.QTSOLIPROD,
                      PRODSOLI.CDPRODESTO,
                      NVL(PRODSOLI.VRFATOCONV, 1) AS VRFATOSOLI,
                      NVL(PRODSOLI.VRARREDCMP, 1) VRARREDSOLI,
                      NVL(PRODUTO.VRFATOCONV, 1) VRFATOCONV,
                      NVL(PRODUTO.VRARREDCMP, 1) VRARREDCMP
                    FROM
                      ITEMSOLI,
                      PRODUTO PRODSOLI,
                      PRODUTO
                    WHERE
                      ITEMSOLI.CDPRODUTO   = PRODSOLI.CDPRODUTO AND
                      ITEMSOLI.CDFILSOLI   = :P_CDFILSOLI       AND
                      ITEMSOLI.DTPROGENTR  = :P_DTPROGENTR      AND
                      PRODUTO.CDPRODUTO    = :P_CDPRODUTO       AND
                      PRODSOLI.CDPRODESTO  = PRODUTO.CDPRODESTO AND
                      NVL(ITEMSOLI.IDSOLIAPROV, 'S') = 'S'
                    ORDER BY
                       ITEMSOLI.NRSOLICMP";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpQtSoliProd($row, $qntd) {
        $query = "update ITEMSOLI
                    set
                      QTSOLIPROD = QTSOLIPROD + :P_QTSOLIPROD
                    where
                      CDFILSOLI        = :P_CDFILSOLI         AND
                      CDFILLAN         = :P_CDFILLAN          AND
                      NRSOLICMP     = :P_NRSOLICMP      AND
                      CDPRODUTO   = :P_CDPRODUTO    AND
                      DTPROGENTR = :P_DTPROGENTR";
            
        $params = array(
            "P_QTSOLIPROD" => $qntd,
            "P_CDFILSOLI"  => $row["CDFILSOLI"],
            "P_CDFILLAN"   => $row["CDFILLAN"],
            "P_NRSOLICMP"  => $row["NRSOLICMP"],
            "P_CDPRODUTO"  => $row["CDPRODUTO"],
            "P_DTPROGENTR" => substr($row["DTPROGENTR"], 0, 10)
        );
        
        $result = $this->entityManager->getConnection()->executeUpdate($query, $params);
        return $result;
    }
    
    public function acmInsMovOpSol($row, $msg, $qtdOper) {
        $param = array(
            'P_CDFILSOLMOV' => $row['CDFILSOLI'],
            'P_CDFILLANMOV' => $row['CDFILLAN'],
            'P_NRSOLIMOV'   => $row['NRSOLICMP'],
            'P_CDPRODUTO'   => $row['CDPRODUTO'],
            'P_DTENTROPER'  => substr($row['DTPROGENTR'], 0, 10),
            'P_CDOPERADOR'  => $this->environment->getCdOperador(),
            'P_QTANTOPER'   => $row['QTSOLIPROD'],
            'P_QTDEPOPER'   => $qtdOper,
            'P_DSJUSTOPER'  => $msg
        );
        
        $query = "INSERT INTO MOVOPSOL
                    (CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV,
                     CDPRODUTO, DTENTROPER, DTOPERSOL,
                     CDOPERADOR, QTANTOPER, QTDEPOPER,
                    DSJUSTOPER)
                    VALUES
                    (:P_CDFILSOLMOV, :P_CDFILLANMOV, :P_NRSOLIMOV,
                     :P_CDPRODUTO, :P_DTENTROPER, SYSDATE,
                     :P_CDOPERADOR, :P_QTANTOPER, :P_QTDEPOPER,
                    :P_DSJUSTOPER)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmDelItemSoli($row) {
        $param = array(
            'P_CDFILSOLI'   => $row['CDFILSOLI'],
            'P_CDFILLAN'    => $row['CDFILLAN'],
            'P_NRSOLICMP'   => $row['NRSOLICMP'],
            'P_CDPRODUTO'   => $row['CDPRODUTO'],
            'P_DTPROGENTR'  => substr($row['DTPROGENTR'], 0, 10)
        );
        
        $query = "DELETE FROM ITEMSOLI
                    where
                      CDFILSOLI        = :P_CDFILSOLI         AND
                      CDFILLAN         = :P_CDFILLAN          AND
                      NRSOLICMP     = :P_NRSOLICMP      AND
                      CDPRODUTO   = :P_CDPRODUTO    AND
                      DTPROGENTR = :P_DTPROGENTR";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function adqAux_Imposto($row) {
        $param = array();
        $param["P_CDFILDEST"]   = $row["CDFILDEST"];
        $param["P_CDFILLANC"]   = $row["CDFILLANC"];
        $param["P_NRAUTOFORN"]  = $row["NRAUTOFORN"];
        $param["P_CDPRODUTO"]   = $row["CDPRODUTO"];
        $param["P_CDMARCA"]     = $row["CDMARCA"];
        $param["P_DTPROGENTAF"] = substr($row["DTPROGENTAF"], 0, 10);
        
        $query = "SELECT
                      COUNT(*) AS CTIMP,
                      CDFILDEST,    CDFILLANC,
                      NRAUTOFORN,   CDPRODUTO,
                      CDMARCA,      DTPROGENTAF,
                      CDIMPOSTO,    VRIMPITMAF,
                      VRPERREDBCAF, VRBASECALCAF,
                      VRALIQITMAF,  VRPARCDEDAF,
                      CDFISCITMAF,  VRADIMPAF,
                      IDIMPDIGAF
                    FROM
                      IMPITMAF
                    WHERE
                      IMPITMAF.CDFILDEST   = :P_CDFILDEST   AND
                      IMPITMAF.CDFILLANC   = :P_CDFILLANC   AND
                      IMPITMAF.NRAUTOFORN  = :P_NRAUTOFORN  AND
                      IMPITMAF.CDPRODUTO   = :P_CDPRODUTO   AND
                      IMPITMAF.CDMARCA     = :P_CDMARCA     AND
                      IMPITMAF.DTPROGENTAF = :P_DTPROGENTAF
                    GROUP BY
                      CDFILDEST,    CDFILLANC,
                      NRAUTOFORN,   CDPRODUTO,
                      CDMARCA,      DTPROGENTAF,
                      CDIMPOSTO,    VRIMPITMAF,
                      VRPERREDBCAF, VRBASECALCAF,
                      VRALIQITMAF,  VRPARCDEDAF,
                      CDFISCITMAF,  VRADIMPAF,
                      IDIMPDIGAF";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmDelImpItm($row) {
        $param = array(
            'OLD_CDFILDEST'     => $row['CDFILDEST'],
            'OLD_CDFILLANC'     => $row['CDFILLANC'],
            'OLD_NRAUTOFORN'    => $row['NRAUTOFORN'],
            'OLD_DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10),
            'OLD_CDPRODUTO'     => $row['CDPRODUTO'],
            'OLD_CDMARCA'       => $row['CDMARCA']
        );
        
        $query = "DELETE FROM IMPITMAF
                    WHERE CDFILDEST = :OLD_CDFILDEST AND
                                   CDFILLANC = :OLD_CDFILLANC AND
                                   NRAUTOFORN = :OLD_NRAUTOFORN AND
                                   DTPROGENTAF = :OLD_DTPROGENTAF AND
                                   CDPRODUTO = :OLD_CDPRODUTO AND
                                   CDMARCA = :OLD_CDMARCA";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function adqExisteItemautf($row) {
        $param = array();
        $param["OLD_CDFILDEST"]  = $row["CDFILDEST"];
        $param["OLD_CDFILLANC"]  = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"] = $row["NRAUTOFORN"];
        $param["OLD_CDPRODUTO"]  = $row["CDPRODUTO"];
        $param["OLD_CDMARCA"]    = $row["CDMARCA"];
        $param["DTPROGENTAF"]    = substr($row["DTPROGENTAF"], 0, 10);
        
        $query = "SELECT ITEMAUTF.QTPROGENTAF
                  FROM AUTOFORNE, ITEMAUTF, FORNECEDOR
                 WHERE AUTOFORNE.CDFILDEST   = ITEMAUTF.CDFILDEST 
                   AND AUTOFORNE.CDFILLANC   = ITEMAUTF.CDFILLANC 
                   AND AUTOFORNE.NRAUTOFORN  = ITEMAUTF.NRAUTOFORN
                   AND AUTOFORNE.CDFORNECED = FORNECEDOR.CDFORNECED
                   AND ITEMAUTF.CDFILDEST     = :OLD_CDFILDEST
                   AND ITEMAUTF.CDFILLANC     = :OLD_CDFILLANC        
                   AND ITEMAUTF.NRAUTOFORN    = :OLD_NRAUTOFORN  
                   AND ITEMAUTF.CDMARCA       = :OLD_CDMARCA         
                   AND ITEMAUTF.CDPRODUTO     = :OLD_CDPRODUTO   
                   AND ITEMAUTF.DTPROGENTAF   = :DTPROGENTAF";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmDelItemSoliRelacAF($row) {
        $param = array(
            'OLD_CDFILDEST'     => $row['CDFILDEST'],
            'OLD_CDFILLANC'     => $row['CDFILLANC'],
            'OLD_NRAUTOFORN'    => $row['NRAUTOFORN'],
            'OLD_DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10),
            'OLD_CDPRODUTO'     => $row['CDPRODUTO'],
            'OLD_CDMARCA'       => $row['CDMARCA']
        );
        
        $query = "delete from ITEMSOLIRELACAF
                    where
                      CDFILDEST = :OLD_CDFILDEST and
                      CDFILLANC = :OLD_CDFILLANC and
                      NRAUTOFORN = :OLD_NRAUTOFORN and
                      CDPRODUTOAF = :OLD_CDPRODUTO and
                      DTPROGENTAF = :OLD_DTPROGENTAF and
                      CDMARCAAF = :OLD_CDMARCA";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmUpdItm($row, $tipo, $msgJust, $itemAf) {
        if($itemAf != null) {
            $qtd = self::parseFloat($itemAf[0]['QTPROGENTAF']) + $row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG'] ? self::parseFloat($row['QTNOVAPROGAF']) : self::parseFloat($row['QTORIG']);
        } else {
            $qtd = $row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG'] ? self::parseFloat($row['QTNOVAPROGAF']) : self::parseFloat($row['QTORIG']);
        }
        
        $param = array(
            'QTPROGENTAF'       => $tipo == 'A' ? $qtd : self::parseFloat($row['QTORIG']),
            'DTPROGENTAF'       => $tipo == 'A' ? substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'QTNOVAPROGAF'      => null,
            'DSJUSTITMAF'       => null,
            'DTNOVAENTRAF'      => null,
            'CDOPERAPROV'       => $this->environment->getCdOperador(),
            'P_JUSTREPROV'      => $msgJust,
            'P_IDTIPOLANITM'    => $tipo == 'A' ? 'M' : null,
            'OLD_CDFILDEST'     => $row['CDFILDEST'],
            'OLD_CDFILLANC'     => $row['CDFILLANC'],
            'OLD_NRAUTOFORN'    => $row['NRAUTOFORN'],
            'OLD_CDMARCA'       => $row['CDMARCA'],
            'OLD_CDPRODUTO'     => $row['CDPRODUTO'],
            'OLD_DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10)
        );
        
        $query = "update ITEMAUTF
                    set
                      QTPROGENTAF    = :QTPROGENTAF, 
                      DTPROGENTAF    = :DTPROGENTAF,
                      QTNOVAPROGAF = :QTNOVAPROGAF,
                      DSJUSTITMAF      = :DSJUSTITMAF,
                      DTNOVAENTRAF = :DTNOVAENTRAF,
                      CDOPERAPROV = :CDOPERAPROV,
                      DTHRAPROVAF = SYSDATE,
                      DSJUSTITMAFRPV  = :P_JUSTREPROV,
                      IDTIPOLANITM    = :P_IDTIPOLANITM
                    where
                      CDFILDEST        = :OLD_CDFILDEST        AND
                      CDFILLANC        = :OLD_CDFILLANC        AND
                      NRAUTOFORN  = :OLD_NRAUTOFORN  AND
                      CDMARCA          = :OLD_CDMARCA         AND
                      CDPRODUTO     = :OLD_CDPRODUTO   AND
                      DTPROGENTAF = :OLD_DTPROGENTAF";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmDelItemAutF($row) {
        $param = array(
            'OLD_CDFILDEST'     => $row['CDFILDEST'],
            'OLD_CDFILLANC'     => $row['CDFILLANC'],
            'OLD_NRAUTOFORN'    => $row['NRAUTOFORN'],
            'OLD_DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10),
            'OLD_CDPRODUTO'     => $row['CDPRODUTO'],
            'OLD_CDMARCA'       => $row['CDMARCA']
        );
        
        $query = "delete from ITEMAUTF
                    where
                      CDFILDEST = :OLD_CDFILDEST and
                      CDFILLANC = :OLD_CDFILLANC and
                      NRAUTOFORN = :OLD_NRAUTOFORN and
                      CDPRODUTO = :OLD_CDPRODUTO and
                      DTPROGENTAF = :OLD_DTPROGENTAF and
                      CDMARCA = :OLD_CDMARCA";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmInsImpItm($row, $imp) {
        $param = array(
            'CDFILDEST'     => $row['CDFILDEST'],
            'CDFILLANC'     => $row['CDFILLANC'],
            'NRAUTOFORN'    => $row['NRAUTOFORN'],
            'CDPRODUTO'     => $row['CDPRODUTO'],
            'CDMARCA'       => $row['CDMARCA'],
            'DTPROGENTAF'   => substr($row['DTPROGENTAF'], 0, 10),
            'CDIMPOSTO'     => $imp['CDIMPOSTO'],
            'VRIMPITMAF'    => $imp['VRIMPITMAF'],
            'VRPERREDBCAF'  => $imp['VRPERREDBCAF'],
            'VRBASECALCAF'  => $imp['VRBASECALCAF'],
            'VRALIQITMAF'   => $imp['VRALIQITMAF'],
            'VRPARCDEDAF'   => $imp['VRPARCDEDAF'],
            'CDFISCITMAF'   => $imp['CDFISCITMAF'],
            'VRADIMPAF'     => $imp['VRADIMPAF'],
            'IDIMPDIGAF'    => $imp['IDIMPDIGAF']
        );
        
        $query = "INSERT INTO IMPITMAF
                    (CDFILDEST,  CDFILLANC, NRAUTOFORN,
                    CDPRODUTO, CDMARCA, DTPROGENTAF,
                    CDIMPOSTO, VRIMPITMAF, VRPERREDBCAF,
                    VRBASECALCAF, VRALIQITMAF, VRPARCDEDAF,
                    CDFISCITMAF, VRADIMPAF, IDIMPDIGAF)
                    SELECT :CDFILDEST,  :CDFILLANC, :NRAUTOFORN,
                    :CDPRODUTO, :CDMARCA, :DTPROGENTAF,
                    :CDIMPOSTO, :VRIMPITMAF, :VRPERREDBCAF,
                    :VRBASECALCAF, :VRALIQITMAF, :VRPARCDEDAF,
                    :CDFISCITMAF, :VRADIMPAF, :IDIMPDIGAF
                    FROM PARAMGERAL
                    WHERE NOT EXISTS(SELECT 1
                                       FROM IMPITMAF I
                                       WHERE I.CDFILDEST   = :CDFILDEST
                                         AND I.CDFILLANC   = :CDFILLANC
                                         AND I.NRAUTOFORN  = :NRAUTOFORN
                                         AND I.CDPRODUTO   = :CDPRODUTO
                                         AND I.CDMARCA     = :CDMARCA
                                         AND I.DTPROGENTAF = :DTPROGENTAF
                                         AND I.CDIMPOSTO   = :CDIMPOSTO)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function sqlWebTunning() {
        $param = array();
        
        $query = "SELECT
                       MAX(NVL(DSREMOTEADDR, 'VIA BD')) DSREMOTEADDR,
                       MAX(NVL(DSREMOTEHOST, 'VIA BD')) DSREMOTEHOST,
                       MAX(NVL(DSSERVERRDM, 'VIA BD'))  DSSERVERRDM,
                       MAX(NVL(DSSERVMETHOD, 'VIA BD')) DSSERVMETHOD,
                       MAX(NVL(DSREMOTEOPER, 'VIA BD')) DSREMOTEOPER
                    FROM
                       SQLWEBTUNNING
                    WHERE
                       NRAUDSID = USERENV('SESSIONID') AND
                       TRUNC(DTSTARTDATE) = (SELECT MAX(TRUNC(DTSTARTDATE))
                                             FROM SQLWEBTUNNING S
                                             WHERE S.NRAUDSID = USERENV('SESSIONID'))";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsMovOpAF($row, $sqlWT, $msgJust, $stIdOperacao) {
        $param = array(
            'CDFILDESMOV' => $row['CDFILDEST'],
            'CDFILLANMOV' => $row['CDFILLANC'],
            'NRAUTOMOV' => $row['NRAUTOFORN'],
            'DTPROGENTMOV' => substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'CDPRODMOV' => $row['CDPRODUTO'],
            'CDMARCMOV' => $row['CDMARCA'],
            'DSJUSTOPERMOV' => $msgJust,
            'CDOPERADOR' => $this->environment->getCdOperador(),
            'DSALTERAF' => $msgJust,
            'IDOPERAMOVAF' => $stIdOperacao,
            'DSRDMUSUMOVAF' => $sqlWT['DSSERVERRDM'],
            'DSENDUSUMOVAF' => $sqlWT['DSREMOTEADDR'],
            'DSNMUSUMOVAF' => $sqlWT['DSREMOTEHOST'],
            'DSMETUSUMOVAF' => $sqlWT['DSSERVMETHOD']
        );
        
        $query = "insert into MOVOPAF
                      (CDFILDESMOV, CDFILLANMOV, NRAUTOMOV,  CDPRODMOV,    CDMARCMOV, DTPROGENTMOV, DTOPERAF,  CDOPERADOR,     DSALTERAF, DSJUSTOPERMOV,
                       IDOPERAMOVAF ,  DSRDMUSUMOVAF, DSENDUSUMOVAF,    DSNMUSUMOVAF, DSMETUSUMOVAF)
                    values
                      (:CDFILDESMOV, :CDFILLANMOV, :NRAUTOMOV, :CDPRODMOV,    :CDMARCMOV, :DTPROGENTMOV, SYSDATE, :CDOPERADOR,    :DSALTERAF,
                       :DSJUSTOPERMOV,      :IDOPERAMOVAF ,    :DSRDMUSUMOVAF, :DSENDUSUMOVAF,  :DSNMUSUMOVAF,      :DSMETUSUMOVAF)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmInsMovOpAF1($row, $sqlWT, $msgJust, $stIdOperacao) {
        $param = array(
            'CDFILDESMOV' => $row['CDFILDEST'],
            'CDFILLANMOV' => $row['CDFILLANC'],
            'NRAUTOMOV' => $row['NRAUTOFORN'],
            'DTPROGENTMOV' => $stIdOperacao == 'E' ? substr($row['DTPROGENTAF'], 0, 10) : substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'CDPRODMOV' => $row['CDPRODUTO'],
            'CDMARCMOV' => $row['CDMARCA'],
            'DSJUSTOPERMOV' => $msgJust,
            'CDOPERADOR' => $this->environment->getCdOperador(),
            'DSALTERAF' => $msgJust,
            'IDOPERAMOVAF' => $stIdOperacao,
            'DSRDMUSUMOVAF' => $sqlWT['DSSERVERRDM'],
            'DSENDUSUMOVAF' => $sqlWT['DSREMOTEADDR'],
            'DSNMUSUMOVAF' => $sqlWT['DSREMOTEHOST'],
            'DSMETUSUMOVAF' => $sqlWT['DSSERVMETHOD']
        );
        
        $query = "insert into MOVOPAF
                      (CDFILDESMOV, CDFILLANMOV, NRAUTOMOV,  CDPRODMOV,    CDMARCMOV, DTPROGENTMOV, DTOPERAF,  CDOPERADOR,     DSALTERAF, DSJUSTOPERMOV,
                       IDOPERAMOVAF ,  DSRDMUSUMOVAF, DSENDUSUMOVAF,    DSNMUSUMOVAF, DSMETUSUMOVAF)
                    values
                      (:CDFILDESMOV, :CDFILLANMOV, :NRAUTOMOV, :CDPRODMOV,    :CDMARCMOV, :DTPROGENTMOV, SYSDATE, :CDOPERADOR,    :DSALTERAF,
                       :DSJUSTOPERMOV,      :IDOPERAMOVAF ,    :DSRDMUSUMOVAF, :DSENDUSUMOVAF,  :DSNMUSUMOVAF,      :DSMETUSUMOVAF)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmInsLogPlanItemAF($row, $msgJust, $stIdOperacao) {
        $param = array(
            'P_NRLOGPLANITEMAF' => self::procedureGeraNovoCodigo('LOGPLANITEMAF'),
            'P_CDFILDEST' => $row['CDFILDEST'],
            'P_CDFILLANC' => $row['CDFILLANC'],
            'P_NRAUTOFORN' => $row['NRAUTOFORN'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_CDMARCA' => $row['CDMARCA'],
            'P_DTPROGENTAFANT' => substr($row['DTPROGENTAF'], 0, 10),
            'P_DTPROGENTAFDEP' => substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'P_DSJUSTOPERMOV' => $msgJust,
            'P_QTPROGENTANT' => self::parseFloat($row['QTORIG']),
            'P_QTPROGENTDEP' => $row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG'] ? self::parseFloat($row['QTNOVAPROGAF']) : self::parseFloat($row['QTORIG'])
        );
        
        $query = "INSERT
                      INTO LOGPLANITEMAF
                           (NRLOGPLANITEMAF, CDFILDEST, CDFILLANC, NRAUTOFORN, CDMARCA, CDPRODUTO,
                           DTPROGENTAFANT, DTPROGENTAFDEP, QTPROGENTANT, QTPROGENTDEP, DSJUSTITMAF, DTOPERAF)
                    VALUES (:P_NRLOGPLANITEMAF, :P_CDFILDEST, :P_CDFILLANC, :P_NRAUTOFORN,:P_CDMARCA, :P_CDPRODUTO,
                            :P_DTPROGENTAFANT, :P_DTPROGENTAFDEP, :P_QTPROGENTANT, :P_QTPROGENTDEP, :P_DSJUSTOPERMOV, SYSDATE)";
                            
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmInsLogPlanItemAF1($row, $msgJust, $stIdOperacao) {
        $param = array(
            'P_NRLOGPLANITEMAF' => self::procedureGeraNovoCodigo('LOGPLANITEMAF'),
            'P_CDFILDEST' => $row['CDFILDEST'],
            'P_CDFILLANC' => $row['CDFILLANC'],
            'P_NRAUTOFORN' => $row['NRAUTOFORN'],
            'P_CDPRODUTO' => $row['CDPRODUTO'],
            'P_CDMARCA' => $row['CDMARCA'],
            'P_DTPROGENTAFANT' => substr($row['DTPROGENTAF'], 0, 10),
            'P_DTPROGENTAFDEP' => $stIdOperacao == 'E' ? null : $stIdOperacao == 'E' ? substr($row['DTPROGENTAF'], 0, 10) : substr($row['DTNOVAENTRAF'], 0, 10) != null && substr($row['DTNOVAENTRAF'], 0, 10) != substr($row['DTPROGENTAF'], 0, 10) ? substr($row['DTNOVAENTRAF'], 0, 10) : substr($row['DTPROGENTAF'], 0, 10),
            'P_DSJUSTOPERMOV' => $msgJust,
            'P_QTPROGENTANT' => self::parseFloat($row['QTORIG']),
            'P_QTPROGENTDEP' => $stIdOperacao == 'E' ? 0 : $row['QTNOVAPROGAF'] != null && $row['QTNOVAPROGAF'] != $row['QTORIG'] ? self::parseFloat($row['QTNOVAPROGAF']) : self::parseFloat($row['QTORIG'])
        );
        
        $query = "INSERT
                      INTO LOGPLANITEMAF
                           (NRLOGPLANITEMAF, CDFILDEST, CDFILLANC, NRAUTOFORN, CDMARCA, CDPRODUTO,
                           DTPROGENTAFANT, DTPROGENTAFDEP, QTPROGENTANT, QTPROGENTDEP, DSJUSTITMAF, DTOPERAF)
                    VALUES (:P_NRLOGPLANITEMAF, :P_CDFILDEST, :P_CDFILLANC, :P_NRAUTOFORN,:P_CDMARCA, :P_CDPRODUTO,
                            :P_DTPROGENTAFANT, :P_DTPROGENTAFDEP, :P_QTPROGENTANT, :P_QTPROGENTDEP, :P_DSJUSTOPERMOV, SYSDATE)";
                            
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function retornaAF($row) {
        $param = array(
            'P_NRAUTOFORN'  => $row['NRAUTOFORN'],
            'P_CDFILDEST'   => $row['CDFILDEST'],
            'P_CDFILLANC'   => $row['CDFILLANC']
        );
        
        $query = "SELECT FORNECEDOR.IDTPENVAFNORMAL, AUTOFORNE.NRSEQENVIOAFORN, AUTOFORNE.CDFILLANC, AUTOFORNE.NRSEQAPURAF, AUTOFORNE.CDFORNECED, AUTOFORNE.CDOPERADOR, AUTOFORNE.NRAUTOFORN, AUTOFORNE.CDFILDEST, AUTOFORNE.DTAUTOFORN DTPROGENTAF, FORNECEDOR.NMRAZSOCFORN, OPERADOR.NMOPERADOR
                      FROM AUTOFORNE, FORNECEDOR, OPERADOR
                     WHERE AUTOFORNE.NRAUTOFORN  = :P_NRAUTOFORN
                       AND AUTOFORNE.CDFILDEST   = :P_CDFILDEST
                       AND AUTOFORNE.CDFILLANC   = :P_CDFILLANC
                       AND AUTOFORNE.CDFORNECED  =  FORNECEDOR.CDFORNECED
                       AND AUTOFORNE.CDOPERADOR  =  OPERADOR.CDOPERADOR";
                       
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqNrApurPrec($row, $cdfilcentcmp) {
        $param = array(
            'P_CDFILCENTCMP'    => $cdfilcentcmp,
            'P_NRAPURPREC'      => $row['NRSEQAPURAF'],
            'P_CDFILLANC'       => $row['CDFILLANC']
        );
        
        $query = "SELECT * FROM APURPRECO WHERE NRAPURPREC = :P_NRAPURPREC AND CDFILCENTCMP = :P_CDFILCENTCMP AND CDFILLANC = :P_CDFILLANC";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqDadosEmail($row, $cdfilcentcmp) {
        $adqNrApurPrec = self::adqNrApurPrec($row, $cdfilcentcmp);
        
        $param = array(
            'P_CDFILCENTCMP'  => $cdfilcentcmp,
            'P_NRSEQAPURAF'   => $row['NRSEQAPURAF'],
            'P_DTINIAPUPREC'  => !empty($adqNrApurPrec) ? isset($adqNrApurPrec[0]['DTINIAPUPREC']) ? substr($adqNrApurPrec[0]['DTINIAPUPREC'], 0, 10) : null : null,
            'P_DTFINAPUPREC'  => !empty($adqNrApurPrec) ? isset($adqNrApurPrec[0]['DTFINAPUPREC']) ? substr($adqNrApurPrec[0]['DTFINAPUPREC'], 0, 10) : null : null
        );
        
        $query = "SELECT DISTINCT APURPRECO.CDFILCENTCMP, APURPRECO.NRAPURPREC, APURPRECO.DTINIAPUPREC, APURPRECO.DTFINAPUPREC,
                       AUTOFORNE.CDFILDEST, AUTOFORNE.CDFILLANC, AUTOFORNE.NRAUTOFORN, F.NMFILIAL, E.IDTPIJURFORN,
                       E.NRINSJURFORN,  E.CDFORNECED, E.NMFANTFORN,
                       E.IDUTILZIPAF, NVL(E.IDTPENVAFNORMAL, 'E') IDTPENVAFNORMAL,
                       E.IDTPENVAFFOR, E.CDFORNERELACAF AS CDFORNERELAC, NVL(E.DSMAILPEDCOLCOP, E.DSMAILCOTCOP) DSMAILCOTCOP,
                       NVL((SELECT LISTAGG(CASE INSTR(REPLACE(NVL(D.DSEMAILECO, D.DSLOGINECO), ';', ','), ',', -1, 1)
                                                     WHEN LENGTH(REPLACE(NVL(D.DSEMAILECO, D.DSLOGINECO), ';', ','))
                                                     THEN SUBSTR(REPLACE(NVL(D.DSEMAILECO, D.DSLOGINECO), ';', ','), 1, INSTR(REPLACE(NVL(D.DSEMAILECO, D.DSLOGINECO), ';', ','), ',', -1, 1) -1)
                                                     ELSE REPLACE(NVL(D.DSEMAILECO, D.DSLOGINECO), ';', ',')
                                                    END, ',') WITHIN GROUP (ORDER BY NVL(D.DSEMAILECO, D.DSLOGINECO)) AS DSEMAILECO
                                       FROM DADOSFORNECOMMERCE D
                                      WHERE D.CDFORNECED  = E.CDFORNECED
                                        AND D.IDACESSOAF  = 'S'), NVL(E.DSEMAILPEDCOLFO, E.DSEMAILCOT)) AS DSEMAILCOT
                
                  FROM ITMAPURPREC  INNER JOIN FILIAL       ON ITMAPURPREC.CDCENTITMAP  = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)
                                                           AND ITMAPURPREC.NRSEQITMAP   = :P_NRSEQAPURAF
                                                           AND ITMAPURPREC.IDITMSELAPUR = 'S'
                                                           AND FILIAL.CDFILIAL          = :P_CDFILCENTCMP
                                    INNER JOIN FORNECEDOR   ON ITMAPURPREC.CDFORNITMAP  = FORNECEDOR.CDFORNECED
                                    INNER JOIN FORNECEDOR E ON NVL(FORNECEDOR.CDFORNERELACAF, FORNECEDOR.CDFORNECED) = E.CDFORNECED
                                    INNER JOIN APURPRECO    ON ITMAPURPREC.CDCENTITMAP  = APURPRECO.CDFILCENTCMP
                                                           AND ITMAPURPREC.NRSEQITMAP   = APURPRECO.NRAPURPREC
                                                           AND APURPRECO.DTINIAPUPREC   = :P_DTINIAPUPREC
                                                           AND APURPRECO.DTFINAPUPREC   = :P_DTFINAPUPREC
                                    INNER JOIN AUTOFORNE    ON ITMAPURPREC.CDFILIAP     = AUTOFORNE.CDFILDEST
                                                           AND ITMAPURPREC.NRSEQITMAP   = AUTOFORNE.NRSEQAPURAF
                                                           AND ITMAPURPREC.CDFORNITMAP  = AUTOFORNE.CDFORNECED
                                    INNER JOIN FILIAL F     ON AUTOFORNE.CDFILDEST      = F.CDFILIAL
                 WHERE NOT EXISTS(SELECT 1
                                    FROM ENVIOAFFORN
                                   WHERE CDFORNECED    = E.CDFORNECED
                                     AND LISTNRSEQAPUR = ITMAPURPREC.NRSEQITMAP
                                     AND DSARQENVIOAF IS NULL
                                     AND IDCONCENVIOAF = 'S')
                 ORDER BY E.CDFORNECED";
                 
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsEnviaAF($dadosEmail, $af, $stNrEnvio, $dtEntrega) {
        $param = array(
            'P_NRSEQENVIOAFORN' => $stNrEnvio,
            'P_CDFORNECED'      => $af['CDFORNECED'],
            'P_DSARQENVIOAF'    => '',
            'P_CDOPERADOR'      => $af['CDOPERADOR'],
            'P_NRAPURPREC'      => $dadosEmail['NRAPURPREC'],
            'P_DTINIAPUPREC'    => substr($dadosEmail['DTINIAPUPREC'], 0, 10),
            'P_DTFINAPUPREC'    => substr($dadosEmail['DTFINAPUPREC'], 0, 10),
            'P_DTENTREGAFFORN'  => substr($dtEntrega, 0, 10),
            'P_QTAFSENVIOFOR'   => 1,
            'P_IDUTILSMTP'      => 'S',
            'P_DSEXTENSAO'      => 'PDF' //ALTERAR FUTURAMENTE
        );
        
        $query = "INSERT
                      INTO ENVIOAFFORN
                           (NRSEQENVIOAFORN, CDFORNECED, CDPAIS, SGESTADO, DTINIAPURAFFORN,
                           DTFINAPURAFFORN, DTENVIOAFFORN, QTAFSENVIOFOR, QTAFSNENVFOR, IDTIPENVLIDO,
                           IDTIPENVAFFORN, DSARQENVIOAF, CDOPERINCLUSAO, DTENTREGAFFORN,
                           LISTNRSEQAPUR, IDUTILSMTP, DSEXTENSAO, IDTCLASSENVAFOR, IDCONCENVIOAF)
                    SELECT :P_NRSEQENVIOAFORN, F.CDFORNECED, E.CDPAIS, E.SGESTADO, :P_DTINIAPUPREC,
                           :P_DTFINAPUPREC, SYSDATE, :P_QTAFSENVIOFOR, 0, 'N',
                           NVL(F.IDTPENVAFNORMAL, 'E'), :P_DSARQENVIOAF, :P_CDOPERADOR, :P_DTENTREGAFFORN,
                           :P_NRAPURPREC, :P_IDUTILSMTP, :P_DSEXTENSAO, 'N', 'N'
                       FROM FORNECEDOR F, ENDEFORN E
                      WHERE F.CDFORNECED   = :P_CDFORNECED
                        AND F.CDFORNECED   = E.CDFORNECED
                        AND E.IDTPENDEFORN = 'P'";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmUpdAutoForne($dado, $stNrEnvio) {
        $param = array(
            'P_NRSEQENVIOAFORN' => $stNrEnvio,
            'P_NRAPURPREC' => $dado['NRAPURPREC'],
            'P_NRAUTOFORN' => $dado['NRAUTOFORN']
        );
        
        $query = "UPDATE AUTOFORNE
                   SET NRSEQENVIOAFORN = :P_NRSEQENVIOAFORN, IDSTATLEITURA = 'N'
                 WHERE INSTR(:P_NRAPURPREC, AUTOFORNE.NRSEQAPURAF) >0
                   AND AUTOFORNE.NRAUTOFORN   = :P_NRAUTOFORN";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmUpdAutoForne1($rows) {
        $param = array(
            'P_CDFILDEST'  => $rows['CDFILDEST'],
            'P_CDFILLANC'  => $rows['CDFILLANC'],
            'P_NRAUTOFORN' => $rows['NRAUTOFORN']
        );
        
        $query = "UPDATE AUTOFORNE
                     SET IDSTATLEITURA = 'N'
                   WHERE CDFILDEST     = :P_CDFILDEST
                     AND CDFILLANC     = :P_CDFILLANC
                     AND NRAUTOFORN    = :P_NRAUTOFORN";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function adqDtEntregAF($dado) {
        $param = array(
            'P_CDFILLANC'   => $dado['CDFILLANC'],
            'P_CDFILDEST'   => $dado['CDFILDEST'],
            'P_NRAUTOFORN'  => $dado['NRAUTOFORN']
        );
        
        $query = "SELECT MIN(ITEMAUTF.DTPROGENTAF) DTPROGENTAF
                      FROM ITEMAUTF
                     WHERE ITEMAUTF.CDFILLANC     = :P_CDFILLANC
                       AND ITEMAUTF.CDFILDEST     = :P_CDFILDEST
                       AND ITEMAUTF.NRAUTOFORN    = :P_NRAUTOFORN";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqDadosRemaneja($dado) {
        $param = array(
            'P_CDFILCENTCMP'    => $dado['CDFILLANC'],
            'P_NRAPURPREC'      => $dado['NRAPURPREC']
        );
        
        $query = "SELECT
                       APURPRECO.IDCONSOLAPUEFET
                    FROM
                       APURPRECO
                    WHERE APURPRECO.CDFILCENTCMP = :P_CDFILCENTCMP
                      AND APURPRECO.NRAPURPREC = :P_NRAPURPREC";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmUpdEnviaAF($af, $dado, $confirmaEnvio) {
        $param = array(
            'P_CDFORNECED'      => $af['CDFORNECED'],
            'P_LISTNRSEQAPUR'   => $af['NRSEQAPURAF'],
            'P_DTINIAPURAFFORN' => $dado['DTINIAPUPREC'],
            'P_DTFINAPURAFFORN' => $dado['DTFINAPUPREC'],
            'P_IDCONCENVIOAF'   => $confirmaEnvio
        );
        
        $query = "UPDATE ENVIOAFFORN
                   SET IDCONCENVIOAF   = :P_IDCONCENVIOAF
                 WHERE CDFORNECED      = :P_CDFORNECED
                   AND LISTNRSEQAPUR   = :P_LISTNRSEQAPUR
                   AND DTINIAPURAFFORN = :P_DTINIAPURAFFORN
                   AND DTFINAPURAFFORN = :P_DTFINAPURAFFORN";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function updLogDownloadAf($row) {
        $param = array(
            'P_NRSEQENVIOAFORN' => $row['NRSEQENVIOAFORN'],
            'P_CDFORNECED'      => $row['CDFORNECED'],
            'P_NRAUTOFORN'      => $row['NRAUTOFORN'],
            'P_CDFILDEST'       => $row['CDFILDEST'],
            'P_CDFILLANC'       => $row['CDFILLANC']
        );
        
        $query = "UPDATE LOGDOWNLOADAF
                  SET DTDOWNLOAD = NULL
                  WHERE  NRSEQENVIOAFORN = :P_NRSEQENVIOAFORN 
                    AND CDFORNECED       = :P_CDFORNECED
                    AND NRAUTOFORN       = :P_NRAUTOFORN
                    AND CDFILDEST        = :P_CDFILDEST
                    AND CDFILLANC        = :P_CDFILLANC";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function adqEmailAprov($row) {
        $param = array(
            'P_CDFILDEST'   => $row['CDFILDEST'],
            'P_NRAUTOFORN'  => $row['NRAUTOFORN']
        );
        
        $query = "SELECT ENDEFILI.CDFILIAL CDOPGESTOEMAF, 'F' TIPO, ENDEFILI.DSEMAILFILI EMAIL, FILIAL.NMFILIAL NMOPERADOR 
                      FROM ENDEFILI, FILIAL
                     WHERE ENDEFILI.CDFILIAL = FILIAL.CDFILIAL
                       AND ENDEFILI.CDFILIAL = :P_CDFILDEST
                       AND NVL(ENDEFILI.IDTPENDEFILI, 'O') = 'P'
                       AND ENDEFILI.DSEMAILFILI IS NOT NULL
                    UNION ALL
                    SELECT DISTINCT CDOPERADORITMAF CDOPGESTOEMAF, 'O' TIPO, OPERADOR.DSEMAILOPER EMAIL, OPERADOR.NMOPERADOR
                      FROM ITEMAUTF, OPERADOR
                     WHERE ITEMAUTF.CDOPERADORITMAF = OPERADOR.CDOPERADOR
                       AND OPERADOR.DSEMAILOPER IS NOT NULL
                       AND ITEMAUTF.CDFILDEST = :P_CDFILDEST
                       AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqEmailGestorAF($row) {
        $param = array(
            'P_CDFILDEST'   => $row['CDFILDEST'],
            'P_NRAUTOFORN'  => $row['NRAUTOFORN']
        );
        
        $query = "SELECT DISTINCT CDOPERADORITMAF CDOPGESTOEMAF, 'O' TIPO, OPERADOR.DSEMAILOPER EMAIL, OPERADOR.NMOPERADOR
                      FROM ITEMAUTF, OPERADOR
                     WHERE ITEMAUTF.CDOPERADORITMAF = OPERADOR.CDOPERADOR
                       AND OPERADOR.DSEMAILOPER IS NOT NULL
                       AND ITEMAUTF.CDFILDEST = :P_CDFILDEST
                       AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqEmailFornecedor($row) {
        $query = "SELECT F.IDTPIJURFORN,
                         F.NRINSJURFORN,
                         F.CDFORNECED,
                         F.NMFANTFORN,
                         F.IDUTILZIPAF,
                         NVL(F.IDTPENVAFNORMAL, 'E') AS IDTPENVAFNORMAL,
                         F.IDTPENVAFFOR,
                         F.CDFORNERELACAF AS CDFORNERELAC,
                         REPLACE(NVL((SELECT DSEMAILFORN FROM TABLE(DADOS_FORN_EMAIL (E.CDFORNECED, 'S', '', E.NRORG))), NVL(E.DSEMAILPEDCOLFO, E.DSEMAILCOT)), ',', ';') AS DSEMAILCOT,
                         REPLACE(NVL(E.DSMAILPEDCOLCOP, E.DSMAILCOTCOP), ',', ';') DSMAILCOTCOP 
                    FROM FORNECEDOR F INNER JOIN FORNECEDOR E ON NVL(F.CDFORNERELACAF, F.CDFORNECED) = E.CDFORNECED
                                                             AND F.NRORG = E.NRORG
                   WHERE F.CDFORNECED = :P_CDFORNECED
                     AND F.NRORG = :NRORG";
                      
        $param = [
            "P_CDFORNECED" => $row['CDFORNECED'],
            "NRORG"        => $this->environment->getNrOrg()
        ];
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            $resultado = $std->fetchAll();
            
            return $resultado;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function retornaLinkEcommerce($row) {   
        $query = "SELECT MAX(DSLINKECOMMERCE) AS DSLINKECOMMERCE
                    FROM PARAMGERALECOMMERCE 
                   WHERE INSTR(:P_CDFILCENTCMP, CDFILIAL) > 0
                     AND NRORG = :NRORG";

        $params = array(
            "P_CDFILCENTCMP" => $row,
            "NRORG"        => $this->environment->getNrOrg()
        );
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
            
            $resultado = $std->fetchAll();
            
            return is_array($resultado) && count($resultado) ? $resultado[0] : null;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    
    public function acmUpdateTpCtPagar($row) {
        $param = array(
            'P_CDFILDEST'       => $row['CDFILDEST'],
            'P_CDFILLANC'       => $row['CDFILLANC'],
            'P_NRAUTOFORN'      => $row['NRAUTOFORN'] 
        );
        
        $query = "UPDATE AUTOFORNE
                    SET CDTPCTPAGAR = (SELECT
                                          MAX(PRODUTO.CDTPCTPAGAR)
                                       FROM
                                          PRODUTO,
                                          (SELECT
                                              MIN(CDPRODUTO) AS CDPRODUTO
                                           FROM
                                              (SELECT
                                                  CDARVPROD CDPRODUTO,
                                                  MAX(QTPROGENTAF*VRPREUNIAF)
                                               FROM
                                                  ITEMAUTF, PRODUTO
                                               WHERE
                                                  ITEMAUTF.CDFILDEST  = :P_CDFILDEST AND
                                                  ITEMAUTF.CDFILLANC  = :P_CDFILLANC AND
                                                  ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN AND
                                                  ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO AND
                                                  EXISTS (SELECT 1
                                                            FROM PRODUTO PROD
                                                           WHERE (INSTR(PRODUTO.CDARVPROD,PROD.CDARVPROD) = 1) AND
                                                                 PROD.CDTPCTPAGAR IS NOT NULL)
                                               GROUP BY
                                                  CDARVPROD
                                                  HAVING
                                                     MAX(QTPROGENTAF*VRPREUNIAF) =
                                                     (SELECT
                                                         MAX(QTPROGENTAF*VRPREUNIAF)
                                                      FROM
                                                         ITEMAUTF, PRODUTO PR
                                                      WHERE
                                                         ITEMAUTF.CDFILDEST  = :P_CDFILDEST   AND
                                                         ITEMAUTF.CDFILLANC  = :P_CDFILLANC   AND
                                                         ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN  AND
                                                         ITEMAUTF.CDPRODUTO  = PR.CDPRODUTO   AND
                                                         EXISTS (SELECT 1
                                                                   FROM PRODUTO PROD
                                                                  WHERE (INSTR(PR.CDARVPROD,PROD.CDARVPROD) = 1) AND
                                                                        PROD.CDTPCTPAGAR IS NOT NULL)
                                                         ))) ITEMAUTF
                                       WHERE
                                          PRODUTO.CDARVPROD = (SELECT MAX(PROD.CDARVPROD)
                                                               FROM PRODUTO PROD
                                                               WHERE
                                                                (INSTR(ITEMAUTF.CDPRODUTO,PROD.CDARVPROD) = 1) AND
                                                                 PROD.CDTPCTPAGAR IS NOT NULL))
                    WHERE
                       AUTOFORNE.CDFILDEST  = :P_CDFILDEST  AND
                       AUTOFORNE.CDFILLANC  = :P_CDFILLANC  AND
                       AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN AND
                       AUTOFORNE.CDTPCTPAGAR IS NULL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }
    
    public function acmUpdateTpCtPagar1($row) {
        $param = array(
            'P_CDFILDEST'       => $row['CDFILDEST'],
            'P_CDFILLANC'       => $row['CDFILLANC'],
            'P_NRAUTOFORN'      => $row['NRAUTOFORN'] 
        );
        
        $query = "UPDATE AUTOFORNE
                     SET CDTPCTPAGAR = NULL
                   WHERE AUTOFORNE.CDFILDEST  = :P_CDFILDEST
                     AND AUTOFORNE.CDFILLANC  = :P_CDFILLANC
                     AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN
                     AND AUTOFORNE.CDTPCTPAGAR IS NOT NULL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch (Exception $e){
            throw $e;
        }
    }

    public function fechamento_af($procParams) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'FECHAMENTO_AF');
        $procedure->addParam(new Param('P_CDFILDEST',   Param::PARAM_INPUT, $procParams["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',   Param::PARAM_INPUT, $procParams["CDFILLANC"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN',  Param::PARAM_INPUT, $procParams["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function exclui_contapagar_prevista($procParams) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'EXCLUI_CONTAPAGAR_PREVISTA');
        $procedure->addParam(new Param('P_CDFILDEST',   Param::PARAM_INPUT, $procParams["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',   Param::PARAM_INPUT, $procParams["CDFILLANC"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN',  Param::PARAM_INPUT, $procParams["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RESULTADO',   Param::PARAM_OUTPUT));
        $procedure->execute();
    }
    
    public function gera_contapagar_prevista($procParams) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'GERA_CONTAPAGAR_PREVISTA');
        $procedure->addParam(new Param('P_CDFILDEST',   Param::PARAM_INPUT, $procParams["CDFILDEST"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILLANC',   Param::PARAM_INPUT, $procParams["CDFILLANC"],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAUTOFORN',  Param::PARAM_INPUT, $procParams["NRAUTOFORN"],  Param::PARAM_TYPE_STR));
        $procedure->execute();
    }

}