<?php
namespace Teknisa\CMP\Modules\Solicitacao\Modules\SolicitacaoCompras\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\CMP\Modules\Solicitacao\Modules\SolicitacaoCompras\Services\CMP03100LancamentoDeSolicitacaoDeComprasService;
use Exception;
use \Setforms2\Shared\ORM\DateTime;

class CMP03100LancamentoDeSolicitacaoDeComprasController extends \Teknisa\GER\Controllers\TeknisaCrud
{
    protected $newCode;
    protected $crudService;
    
    public function __construct($crudService, $parameterBag, CMP03100LancamentoDeSolicitacaoDeComprasService $service, $environment, $newCode, $dataprocessamento) 
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
		$this->service      = $service;
		$this->environment  = $environment;
		$this->newCode      = $newCode;
		$this->dataprocessamento = $dataprocessamento;
	}
	
	public function retornaFloat($valor) {
	    
	    if(gettype($valor) === 'string') {
	        
	        $valor = str_replace('.', '', $valor);
            $valor = str_replace(',', '.', $valor);
	        
	        $valor = (float) $valor;
	    }
	    
	    return $valor;
	    
	}
	
	public function completaProd($campo)
    {
        return str_pad($campo, 20, 'Z', STR_PAD_RIGHT);
    }
	
	public function retornaCentralCompras(DTO\Request\Row $request, DTO\Response $response) 
	{
	    
	    $row = $request->getRow();
	    
	    $centralCompras = $this->service->retornaCentralCompras($row['CDFILIAL']);
	    
	    $response->addDataSet(new DataSet('centralCompras', $centralCompras));
	    
	}
	
	public function validaFilial(DTO\Request\Row $request, DTO\Response $response) {
	    $filial = $request->getRow();
	    $dataSet = $this->service->getUtDestSol($filial[0]);
        
        $response->addDataSet(new DataSet('response', array($dataSet))); 
    }
    
    public function filiaisSelecionadas(DTO\Request\Row $request, DTO\Response $response) {
	    $filial         = $request->getRow();
        $filialDataSet  = $this->service->filiaisSelecionadas($filial);
        
        $response->addDataSet(new DataSet('res', array($filialDataSet))); 
    }
    
    public function validaGerente(DTO\Request\Row $request, DTO\Response $response){
        $row        = $request->getRow();
        
        $aprovacao  = $this->service->validaAprovacao();
        
        if($aprovacao[0]['CONTADOR'] > 0){
            $gerDataSet = $this->service->validaGerente($row[0], $row[1]);
            if(empty($gerDataSet['CDOPERADOR'])){
                $response->addDataSet(new DataSet('ger', array($aprovacao[0]['CONTADOR'], 0)));
            }else{
                $response->addDataSet(new DataSet('ger', array($aprovacao[0]['CONTADOR'], $gerDataSet[0])));
            }
        }else{
            $response->addDataSet(new DataSet('ger', array($aprovacao[0]['CONTADOR'])));
        }
    }
    
    public function selectFiltro(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    if($row['TENTREGA'] == 'T'){
            $dataFiltro = $this->service->selectFiltro($row);
	    }else{
	        $dataFiltro = $this->service->selectFiltroEntrega($row);
	    }
        
        $response->addDataSet(new DataSet('res', array($dataFiltro))); 
    }
    
    public function selectFiltroServ(DTO\Request\Row $request, DTO\Response $response) {
	    $row            = $request->getRow();
        $dataFiltroServ = $this->service->selectFiltroServ($row);
        
        $response->addDataSet(new DataSet('res', array($dataFiltroServ))); 
    }
    
    public function selectSoliciFilial(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
        $solicitacoes = $this->service->selectSoliciFilial($row);
        
        $response->addDataSet(new DataSet('res', array($solicitacoes))); 
    }
    
    public function getSolicitacao(DTO\Request\Row $request, DTO\Response $response) {
	    $row                = $request->getRow();
        $justSolicitacao    = $this->service->getJustSolicitacao($row);
        $valorSolicitacao   = $this->service->getValorSolicitacao($row);
        
        $response->addDataSet(new DataSet('res', array($justSolicitacao, $valorSolicitacao)));
    }
    
    public function deleteSolicitacao(DTO\Request\Row $request, DTO\Response $response) {
         try {
	        $row = $request->getRow();
	        
	        if(!empty($row['NMPATHANEXO'])) {
	            //delete file in FileServer and the row on table ANEXOSOLICITA
	            $this->service->deleteAnexo($row);
	        }
	        
	        $this->service->deleteSolicitacao($row);
            $this->service->deleteSolicitacaoGer($row);
            
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function selectProdutos(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    $verifyProd = $this->service->decisionProdutos($row);
        
            if($verifyProd[0]['TOTAL'] == 1){
                $oneRecord = 'S';
            }else{
                $oneRecord = 'N';
            }
        $filialDataSet = $this->service->selectProdutos($row, $oneRecord);
        
        $response->addDataSet(new DataSet('res', array($filialDataSet))); 
        
    }
    
    public function selectDetalhesProduto(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    
	    $verifyProd = $this->service->decisionProdutos($row);
        
        if($verifyProd[0]['TOTAL'] == 1){
            $oneRecord = 'S';
        }else{
            $oneRecord = 'N';
        }
        $filialDataSet = $this->service->selectDetalhesProduto($row, $oneRecord);
        $response->addDataSet(new DataSet('res', array($filialDataSet)));
    }
    
    public function verificaDetalhesProduto(DTO\Request\Row $request, DTO\Response $response)
    {
	    try {
            $row  = $request->getRow();
    	    $count = !is_null($row) ? count($row) : 0;
    	    
    	    if (!$count) {
    	        throw new \Exception('Não existem itens selecionados para realizar essa operação.', 500);
    	    }
    	    
            self::permiteAlterarSolicitacaoAutomatica($row[0], 'EXCLUIR_PRODUTO');
            
            for ($i = 0; $i < $count; $i++) {
    	        self::deleteProduto($row[$i]);
            }
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function deleteProduto($row) {
	    $verifyProd = $this->service->decisionProdutos($row);
        
        if($verifyProd[0]['TOTAL'] == 1){
            $oneRecord = 'S';
        }else{
            $oneRecord = 'N';
        }
        
        $filialDataSet = $this->service->selectDetalhesProduto($row, $oneRecord);
        
        $count = !is_null($filialDataSet) ? count($filialDataSet) : 0;
        
        for($k = 0; $k < $count; $k++){
	        $filialDataSet[$k]['CDPRODUTO'] = $row['CDPRODUTO'];
            
            $prodCompra = $this->service->queryProdCompra($filialDataSet[$k]);
	        
            if(!empty($prodCompra)){
                $this->service->delForneced($filialDataSet[$k]);
                $this->service->delProdCompra($filialDataSet[$k]);
                $this->service->updCmp($filialDataSet[$k]);
            }else{
                $this->service->delItemSoli($filialDataSet[$k]);
                $existeAprovSoliGer = $this->service->queryExisteAprovSoliGer($filialDataSet[$k]);
                if(empty($existeAprovSoliGer)){
                    $this->service->delAprovSoliGer($filialDataSet[$k]);
                }
            }
            
    	    $filialDataSet[$k]['DSJUSTOPER'] = $row['DSJUSTOPER'];
    	    $filialDataSet[$k]['OLD_QTSOLIPROD'] = self::retornaFloat($filialDataSet[$k]['QTSOLIPROD']);
            $filialDataSet[$k]['QTSOLIPROD'] = 0;
            $filialDataSet[$k]['IDOPERAMOVS'] = 'E';
            $filialDataSet[$k]['CDOPERADOR'] = $this->environment->getCdOperador();
            
            if($row['DSJUSTOPER'] != null){
                $this->service->insMovoPSol($filialDataSet[$k]);
            }
            
        }    
    }
    
    public function getJustSolicitacao(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
        $filialDataSet = $this->service->getJustSolicitacao($row);
        $response->addDataSet(new DataSet('res', array($filialDataSet))); 
    }
    
    public function getValorSolicitacao(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
        $valorSolicitacao = $this->service->getValorSolicitacao($row);
        $response->addDataSet(new DataSet('res', array($valorSolicitacao))); 
    }
    
    public function salvarDataProd(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            $items = $dataset[1];
            
            //self::permiteAlterarSolicitacaoAutomatica($row, 'INCLUIR_DTENTREGA');
            $paramFilial = $this->service->getParamFilial($row);
            
            self::incluirDataProd($items, $paramFilial);
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function incluirDataProd($items, $paramFilial) {
        foreach ($items as $row) {
            if(isset($row['QTSOLIPROD'])) {
                $row['QTSOLIPROD'] = self::retornaFloat($row['QTSOLIPROD']);
                
                //$row['QTVOLUITEMSO'] = self::retornaFloat($row['QTVOLUITEMSO']);
            
                if($row['QTSOLIPROD'] == 0) {
                    $row['IDOPERAMOVS'] = 'E';
                    $ProdCompra = $this->service->queryProdCompra($row);
                    
                    if(!empty($ProdCompra['NRCOMPRA'])) {
                        $this->service->delForneced($row);
                        $this->service->delProdCompra($row);
                        $this->service->updCmp($row);
                    }
                    
                    $this->service->delItemSoli($row);
                    
                    $existeAprovSoliGer = $this->service->queryExisteAprovSoliGer($row);
                    
                    if(empty($existeAprovSoliGer)){
                        $this->service->delAprovSoliGer($row);
                    }
                } else {
                    $row['IDOPERAMOVS'] = 'I';
        
                    $this->service->insItemSoli($row);
                    
                    if($paramFilial[0]['IDUTILIZAINTSAC'] == 'S'){
                        $this->service->updSolicitaInt($row);
                    }
                }
            }
            // if(isset($row['DSJUSTOPER'])) {
            //     if($row['DSJUSTOPER'] != null) {
            //         $this->service->insMovoPSol($row);
            //     }
            // }
        }
    }
    
    private function verificaDataEntrega($row)
    {
        $result = $this->dataprocessamento->getDataprocessamento($row["CDFILSOLI"]);
        $novaData = DateTime::createFromFormat("d/m/Y", $row["DTPROGENTR"]);
    	if($novaData->format('Y/m/d') < $result) {
            throw new Exception('DATAPROC_MAIOR');       
        }
    }
    
    public function salvarDataProdNovo(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	        $row = $request->getRow();
    	    $count = !is_null($row) ? count($row) : 0;
    	    
    	    if (!$count) {
    	        throw new \Exception('Não existem itens selecionados para realizar essa operação.', 500);
    	    }
    	    
    	    //self::permiteAlterarSolicitacaoAutomatica($row[0], 'INCLUIR_PRODUTO');
    	    
    	    for ($i = 0; $i < $count; $i++) {
    	        self::incluirDataProdNovo($row[$i]);
            }
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function incluirDataProdNovo($row) {
        $paramFilial = $this->service->getParamFilial($row);
        
        $row['QTSOLIPROD'] = self::retornaFloat($row['QTSOLIPROD']);
        //$row['QTVOLUITEMSO'] = self::retornaFloat($row['QTVOLUITEMSO']);
        
        if($row['QTSOLIPROD'] == 0){
            $row['IDOPERAMOVS'] = 'E';
            $ProdCompra = $this->service->queryProdCompra($row);
            
            if(!empty($ProdCompra['NRCOMPRA'])){
                $this->service->delForneced($row);
                $this->service->delProdCompra($row);
                $this->service->updCmp($row); 
            }
            
            $this->service->delItemSoli($row);
            
            $existeAprovSoliGer = $this->service->queryExisteAprovSoliGer($row);
            if(empty($existeAprovSoliGer)){
                $this->service->delAprovSoliGer($row);
            }
        }else{
            $row['IDOPERAMOVS'] = 'I';
            $this->service->insItemSoli($row);
            
            if($paramFilial[0]['IDUTILIZAINTSAC'] == 'S'){
                $this->service->updSolicitaInt($row);
            }
        }
        
        if($row['DSJUSTOPER'] !== null){
    	    $row['OLD_QTSOLIPROD'] = 0;
    	    $row['IDOPERAMOVS'] = 'I';
    	    $row['QTSOLIPROD'] = self::retornaFloat($row['QTSOLIPROD']);
            $this->service->insMovoPSol($row);
        }
    }
    
    private function setErroLog(&$error, $nmproduto, $qtde, $data) {
        $error[] = array(
            "MESSAGE" => "QTSOLIPROD_CANNOT_BE_CHANGED", 
            "NMPRODUTO" => $nmproduto,
            "QTSOLIPROD" => $qtde,
            "DTPROGENTR" => $data
        );
        //$error[] = array("MESSAGE" => "WITHOUT_PERMISSION");
    }
    
    public function salvarQuantidadeProd(DTO\Request\Row $request, DTO\Response $response) {
        $error = array();
        
        try {
            $rows = $request->getRow();
            
            if(sizeof($rows)) {
                $exigePermissaoAlteracaoSolicitacaoAutomatica = self::exigePermissaoAlteracaoSolicitacaoAutomatica($rows[0]);
                if($exigePermissaoAlteracaoSolicitacaoAutomatica) {
                    $validaAcaoAumentar = self::operadorPossuiPermissao('AUMENTAR_QUANTIDADE');
                    $validaAcaoDiminuir = self::operadorPossuiPermissao('DIMINUIR_QUANTIDADE');
                    $permissoes = $this->service->queryAutGrSoli();
                    $percentAumento = 0;
                    $percentReducao = 0;
                            
                    foreach ($permissoes as $value) {
                        if ($value['IDAUTLANCSOLI'] == 'AUMENTAR_QUANTIDADE') {
                            $percentAumento = $value['PEALTEQTDESOLI'];
                        }
                        
                        if ($value['IDAUTLANCSOLI'] == 'DIMINUIR_QUANTIDADE') {
                            $percentReducao = $value['PEALTEQTDESOLI'];
                        }
                    }
                }
            } else {
                $error[] = array("MESSAGE" => "WITHOUT_SELECTED_ITEMS");
            }
            
            foreach ($rows as $row) {
                $row['CDFILIAL'] = $row['CDFILSOLI'];
                $row['CDFILLANCA'] = $row['CDFILLAN'];
                $strQTSOLIPROD = $row['QTSOLIPROD'];
        	    
    	        if ($exigePermissaoAlteracaoSolicitacaoAutomatica) {
    	            $validaAcaoAumentar = self::operadorPossuiPermissao('AUMENTAR_QUANTIDADE');
                    $validaAcaoDiminuir = self::operadorPossuiPermissao('DIMINUIR_QUANTIDADE');
    	            $qtOriginal = floatval($row['changed_QTSOLIPROD'] == 0 || $row['changed_QTSOLIPROD'] == null ? $row['changed_QTSOLIPROD'] : $row['QTSOLIORIG']);
                    $row['QTSOLIPROD'] = self::retornaFloat($row['QTSOLIPROD']);
                    $row['changed_QTSOLIPROD'] = self::retornaFloat($row['changed_QTSOLIPROD']);
                    
                    if ($row['QTSOLIPROD'] > $row['changed_QTSOLIPROD']) {
                        if ($validaAcaoAumentar) {
                            if ($percentAumento === null) {
                                self::updateQtdProduto($row);    
                            } else {
                                if($percentAumento === 0) {
                                    $this->setErroLog($error, $row['NMPRODUTO'], $strQTSOLIPROD, $row["DTPROGENTR"]);
                                } else {
                                    if ($row['QTSOLIPROD'] > $row['changed_QTSOLIPROD'] + ($row['changed_QTSOLIPROD'] * ($percentAumento / 100))) {
                                        $this->setErroLog($error, $row['NMPRODUTO'], $strQTSOLIPROD, $row["DTPROGENTR"]);
                                    } else {
                                        self::updateQtdProduto($row);
                                    }
                                }
                            }
                        } else {
                            $this->setErroLog($error, $row['NMPRODUTO'], $strQTSOLIPROD, $row["DTPROGENTR"]);
                        }
                    }
                    
                    if ($row['QTSOLIPROD'] < $row['changed_QTSOLIPROD']) {
                        if ($validaAcaoDiminuir) {
                            if ($percentReducao === null) {
                                self::updateQtdProduto($row);    
                            } else {
                                if($percentReducao === 0) {
                                    $this->setErroLog($error, $row['NMPRODUTO'], $strQTSOLIPROD, $row["DTPROGENTR"]);
                                } else {
                                    if ($row['QTSOLIPROD'] < $row['changed_QTSOLIPROD'] - ($row['changed_QTSOLIPROD'] * ($percentReducao / 100))) {
                                        $this->setErroLog($error, $row['NMPRODUTO'], $strQTSOLIPROD, $row["DTPROGENTR"]);
                                    } else {
                                        self::updateQtdProduto($row);
                                    }
                                }
                            }
                        } else {
                            $this->setErroLog($error, $row['NMPRODUTO'], $strQTSOLIPROD, $row["DTPROGENTR"]);
                        }
                    }
                } else {
                    self::updateQtdProduto($row);  
                }
            }
            
            $response->addMessage(new Message($error, 'W'));
            
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function operadorPossuiPermissao($per) {
        $permissoes = $this->service->queryAutGrSoli();

        foreach ($permissoes as $key => $value) {
            if ($value['IDAUTLANCSOLI'] === $per) {
                return true;
            }
        }
        
        return false;
    }
    
    public function updateQtdProduto($row) {
        $paramFilial = $this->service->getParamFilial($row);
        
        //$row['QTVOLUITEMSO'] = self::retornaFloat($row['QTVOLUITEMSO']);
        $row['QTSOLIPROD'] = self::retornaFloat($row['QTSOLIPROD']);
        if(isset($row['changed_QTSOLIPROD']))
            $row['OLD_QTSOLIPROD'] = self::retornaFloat($row['changed_QTSOLIPROD']);
        else if(isset($row['QTSOLIPROD_inline']))
            $row['OLD_QTSOLIPROD'] = self::retornaFloat($row['QTSOLIPROD_inline']);
        else
            $row['OLD_QTSOLIPROD'] = self::retornaFloat($row['QTSOLIPROD']);
        
        if($row['QTSOLIPROD'] == 0) {
            $row['IDOPERAMOVS'] = 'E';
            $ProdCompra = $this->service->queryProdCompra($row);
            
            if(!empty($ProdCompra['NRCOMPRA'])) {
                $this->service->delForneced($row);
                $this->service->delProdCompra($row);
                $this->service->updCmp($row);
            }
            
            $this->service->delItemSoli($row);
            
            $existeAprovSoliGer = $this->service->queryExisteAprovSoliGer($row);
            
            if(empty($existeAprovSoliGer)) {
                $this->service->delAprovSoliGer($row);
            }
        } else {
            $row['IDOPERAMOVS'] = 'I';
            
            if($paramFilial[0]['IDUTILIZAINTSAC'] == 'S') {
                $this->service->updItSoliInt($row);
                $this->service->updSolicitaInt($row);
            } else {
                if($row['QTSOLIORIG'] == null || $row['QTSOLIORIG'] == 0) {
                    $row['QTSOLIORIG'] = $row['QTSOLIPROD'];
                }
                $this->service->updItemSoli($row);
            }
        }
        
        if($row['DSJUSTOPER'] !== null && $row['QTSOLIPROD'] !== $row['OLD_QTSOLIPROD']) {
            $this->service->insMovoPSol($row);
        }
    }
    
    public function salvaJustificativa(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        
        $this->service->updJustMovoPSol($row);
    }
    
    public function deleteQuantidadeProd(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	        $row = $request->getRow();
    	    $count = !is_null($row) ? count($row) : 0;
    	    
    	    if (!$count) {
    	        throw new \Exception('Não existem itens selecionados para realizar essa operação.', 500);
    	    }
    	    
            //self::permiteAlterarSolicitacaoAutomatica($row[0], 'EXCLUIR_DTENTREGA');
            
            for ($i = 0; $i < $count; $i++) {
                self::deleteDataProd($row[$i]);
            }
            
            $verifyProd = $this->service->decisionProdutos($row[0]);
        
            if($verifyProd[0]['TOTAL'] === 0) {
                $this->service->deleteSolicitacao($row[0]);
                $this->service->deleteSolicitacaoGer($row[0]);
            }
            
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function deleteDataProd($row) {
	    
        $prodCompra = $this->service->queryProdCompra($row);
        
        if(!empty($prodCompra)){
            $this->service->delForneced($row);
            $this->service->delProdCompra($row);
            $this->service->updCmp($row);
        }else {
            $this->service->delItemSoli($row);
            $existeAprovSoliGer = $this->service->queryExisteAprovSoliGer($row);
            if(empty($existeAprovSoliGer)){
                $this->service->delAprovSoliGer($row);
            }
        }
        
        if($row['DSJUSTOPER'] != null){
    	    $row['OLD_QTSOLIPROD'] = self::retornaFloat($row['QTSOLIPROD']);
    	    $row['QTSOLIPROD'] = 0;
    	    $row['IDOPERAMOVS'] = 'E';
    	    $this->service->insMovoPSol($row);
        }
    }
    
    public function salvaSolicitacao(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();

        if($row['__isNew'] == true) {
	        $NRSOLICMP = $this->newCode->getCodeByTableName('SOLICOMP' . $row['CDFILSOLI'] . $row['CDFILLANCA'], $this->environment->getNrorg());
	        
            $this->service->insSolicita($row, $NRSOLICMP);
            $this->service->insJustSoli($row, $NRSOLICMP);
        } else {
            $NRSOLICMP = $row['NRSOLICMP'];
            $existeSolicita = $this->service->getJustSolicitacao($row);
            $this->service->updSolicita($row);
            if(empty($existeSolicita)) {
                $this->service->insJustSoli($row, $row['NRSOLICMP']);
            } else {
                $this->service->updJustSoli($row);
            }
        }
        
        $response->addDataSet(new DataSet('sol', array($NRSOLICMP))); 
    }
    
    public function listaProdutos(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	        $row = $request->getRow();
	        
	        $row['CDPRODFIN'] = self::completaProd($row['CDPRODFIN']);
	        
    	    if(!$row['CDFILIAL_SOL'] || $row['CDFILIAL_SOL'] == "") {
    	        $filialDataSet = $this->service->listaProdutos($row);
    	    } else {
    	        $filialDataSet = $this->service->listaProdutosPorSolicitacao($row);
    	    }
    	    if(!$filialDataSet)
    	        throw new Exception("NAO_HA_PROD_CADASTRADO_UNIDADE");
            
            $response->addDataSet(new DataSet('res', array($filialDataSet))); 
	        
	    } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function getProdutos(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    $produto = $this->service->getProdutoMin($row);
        
        $response->addDataSet(new DataSet('res', array($produto))); 
    }
    
    public function validarInclusProdCompra(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    
	    $validaFilial = $this->service->validaUnidade($row['CDFILSOLI']);
	    if(!empty($validaFilial)){
                
            $filialDataSet = $this->service->validarInclusProdCompra($row);
            if(!empty($filialDataSet)){
                $response->addDataSet(new DataSet('res', array($filialDataSet))); 
            }else{
                $erro = new Error("Não há parâmetros de compras para a central de compras da unidade!", 500);
                $response->setError($erro);
            }
            
	    }else{
	        $erro = new Error("Unidade com contrato rescindido!", 500);
            $response->setError($erro);
	    }
    }
    
    public function getPermissoes(DTO\Request\Row $request, DTO\Response $response) {
        $permissoes = $this->service->queryAutGrSoli();
        
        $response->addDataSet(new DataSet('permissoes', $permissoes)); 
    }
    
    public function validaTipoCusto(DTO\Request\Row $request, DTO\Response $response) 
    {
	    $row = $request->getRow();
	    
        $tipoCusto = $this->service->getTipoCusto($row);
        
        $response->addDataSet(new DataSet('cust', $tipoCusto)); 
    }
    
    public function selectProdutosGrid(DTO\Request\Row $request, DTO\Response $response) 
    {
	    $row = $request->getRow();
        
        $produtos = $this->service->selectProdutosGrid($row);
        $response->addDataSet(new DataSet('produtos', $produtos)); 
    }
    
    public function selectCalculaMedia(DTO\Request\Row $request, DTO\Response $response) 
    {
	    $row = $request->getRow();
        
        $media= $this->service->selectCalculaMedia($row);
        $response->addDataSet(new DataSet('media', $media)); 
    }
    
    public function existeProdutosAprovados(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        
        $qtdeProdutosAprovados = $this->service->getQuantidadeProdutosAprovadosNaSolicitacao($row)['CONTAPROVACAO'];
        
        if ($qtdeProdutosAprovados > 0) {
            $dataSetResponse['EXISTE_PRODUTOS_APROVADOS'] = true;
        } else {
            $dataSetResponse['EXISTE_PRODUTOS_APROVADOS'] = false;
        }
        
        $response->addDataSet(new DataSet('data', $dataSetResponse)); 
    }
    
    public function permiteAlterarSolicitacaoAutomatica($row, $permissao) {
        if (!self::exigePermissaoAlteracaoSolicitacaoAutomatica($row)) {
            return false;
        }
        
        if (!self::operadorPossuiPermissao($permissao)) {
            throw new \Exception('Devido ao parâmetro "Bloqueia Alterações nas Solicitações Automáticas" o operador não tem permissão para concluir o procedimento.', 500);
        }
        
        return true;
    }
    
    public function exigePermissaoAlteracaoSolicitacaoAutomatica($row)
    {
        $getParams = $this->service->validarInclusProdCompra($row)[0];
        
        if ($getParams['IDBLOQALTSOLAUT'] == 'N') {
            return false;
        }
        
        $getIdTiPlan = $this->service->getIdTiPlan($row)[0];
        
        if ($getIdTiPlan['IDTIPLAN'] != 'A' && $getIdTiPlan['IDTIPLAN'] != 'C') {
            return false;
        }
        
        return true;
    }
    
    public function getItensSolicitacao(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $parentDataset = $this->crudService->getFilteredDataSet("itens_solicitacao", $conditions)->getRows();
            $sonDataset = array();
            
            foreach ($parentDataset as $row) {
                $newConditions = $conditions;
                $produto = array(
                    "columnName"  => "P_CDPRODUTO",
                    "operator"     => "=",
                    "value"    => $row['CDPRODUTO']
                );
                array_push($newConditions, $produto);
                $dataset = $this->crudService->getFilteredDataSet("details_itens_solicitacao", $newConditions)->getRows();
                $sonDataset = array_merge($sonDataset, $dataset);
            }
            
            $parentDataset = array_values($parentDataset);
            $sonDataset = array_values($sonDataset);
            $dataset = array_merge($parentDataset, $sonDataset);
            $response->addDataSet(new DataSet("itemsoli", $dataset));
        } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function aprovaSolicitacoes(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
	        $row = $request->getRow();
	        $this->service->setAprovaItens($row);
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
        
    }
    
    public function reprovaSolicitacoes(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
	        $row = $request->getRow();
	        $this->service->setReprovaItens($row);
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
        
    }
    
    public function getPermissaoAprovReprovacao(DTO\Request\Row $request, DTO\Response $response) {
        try {
	        $dataset = $this->service->getPermissaoAprovReprovacao();
	        $response->addDataSet(new DataSet("dataset", $dataset));
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
        
    }
    
    public function validaDataProcessamento(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $filialDataSet = $this->service->validaDataProcessamento($row);
        if(!empty($filialDataSet)) {
            $response->addDataSet(new DataSet('res', array($filialDataSet))); 
        } else {
            $erro = new Error("Não há uma Data de Processamento para esta Filial", 500);
            $response->setError($erro);
        }
    }
    
    public function cmp03100TipoCusto(DTO\Request\Row $request, DTO\Response $response) {
        try {
	        $dataset = $this->service->cmp03100TipoCusto();
	        $response->addDataSet(new DataSet("dataset", $dataset));
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function cmp03100ParamCmpSup(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
	        $dataset = $this->service->cmp03100ParamCmpSup($row["P_CDFILIAL"]);
	        $response->addDataSet(new DataSet("dataset", $dataset));
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), 500));
	    }
    }
    
    public function getSolicitacaoParameters(DTO\Request\Row $request, DTO\Response $response) {
        $parametros = $this->service->getSolicitacaoParameters();
        
        $response->addDataSet(new DataSet('parametros', $parametros)); 
    }
    
    /**
     * Busca a url do FileServer
     * @param Row $row
     * @param Response $response
     */
    public function getFileServerUrl(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $fileName = $this->service->getFileServerUrl();
            
            $response->addDataset(new DataSet('sucess', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validaSegMercadoProdFili(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $cdSegMerc = $this->service->getCdSegMercFilial($row["CDFILDEST"]);
            
            if($cdSegMerc) {
                $res = $this->service->verificaSegMercado($row);
                
                if(empty($res) || count($res) == 0) {
                    throw new \Exception("PRODUTO_NOT_SEGMERCADO", 500);
                }
            }
            
            $res1 = $this->service->verificaProdFiliAtivo($row);
            
            if(empty($res1) || count($res1) == 0) {
                throw new \Exception("PRODUTO_NOT_PRODFILI_ATIVO", 500);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}