/*jshint esversion: 6 */
"use strict";

module.exports = function (grunt) {
    var path = require("path");
    var fs = require("fs");
    var shell = require('shelljs');

    var ERROR_TYPE = "ERROR";
    var ABORT_TYPE = "ABORT";
    var SUCCESS = 'success';
    var GREEN = 'green';
    var JSON_PATH = "../mobile/assets/json";
    var SRC_PATH = "../mobile/assets";
    var MODULE_NAME = "product";
    var BUILD_PATH = "./tests";
    var CUSTOM = "/custom/";
    var BUILDED_PATH = '/base/';
    var dontForce = true;

    function buildCustomContent(name) {
        return `/* YOU SHOULD CONFIGURE THIS FILE */
//var ZeedhiAPIConstructor = require('zeedhi-functional-test-api');

/** @name {ZeedhiFunctionalTestApi} z */
//var z = new ZeedhiAPIConstructor(browser, protractor);

//describe("${name}", function(){
//    beforeEach(function(){
//        z.application.openOnContainer('${name}');
//    });

//});

var defaultFilterRow = {};

module.exports = {
    'getDefaultFilterRow' : defaultFilterRow,
    'applyFilterLabel': "Aplicar filtro"
}`;
    }

    function buildContent(name, filter, buildPath) {
        var custom = '/' + name.replace('#', '') + ".spec.js";
        var includeLib = '';
        var includeIt = '';
        var relativeDir = buildPath.replace(BUILDED_PATH, '');

        var content = `/* Warning! This file will always be generated. */
var ZeedhiAPIConstructor = require('zeedhi-functional-test-api');
/** @name {ZeedhiFunctionalTestApi} z */
var z = new ZeedhiAPIConstructor(browser, protractor);

${includeLib}
describe("${name}", function(){
    beforeEach(function(){
        z.application.openOnContainer('${name}');
    });

    it("Default test to check nofitication errors.", function () {
        browser.findElements(by.css('.zh-notification-messages > li')).then(function (elements) {
            var notificationErrorsCounter = 0;
            var promises = [];
            elements.forEach(function(element) {
                promises.push(element.getText().then(function (text) {
                    if (!text.includes("O arquivo que con")) {
                        notificationErrorsCounter++;
                    }
                }));
            });
            Promise.all(promises).then(function(){
                expect(notificationErrorsCounter).toBe(0);
            });
        });
    });
${includeIt}
});`;
        return content;
    }

    function printGruntLog(message, type) {
        var identificationMessage = type && type != ABORT_TYPE ? "\x1b[31m" + type + "\x1b[0m " : "";
        identificationMessage += message;

        if (type == ABORT_TYPE) {
            grunt.warn(identificationMessage);
        } else if (type == ERROR_TYPE) {
            grunt.log.error(identificationMessage);
        } else if (type == SUCCESS) {
            grunt.log.writeln(message[GREEN]);
        } else {
            grunt.log.writeln(identificationMessage);
        }
    }

    function validateOptions(options) {
        var parameterName;
        var parameterType;
        if (typeof options.buildPath !== 'string') {
            parameterName = 'buildPath';
            parameterType = 'string';
        }

        if (parameterName && parameterType) {
            printGruntLog("Invalid arguments: '" + parameterName + "' parameter should be " + parameterType + ".", ABORT_TYPE);
        }
    }

    function iterateFolders(dir, files_, relative_) {
        files_ = files_ || [];
        relative_ = relative_ || [];
        var files = fs.readdirSync(dir);
        for (var i in files) {
            var name = dir + '/' + files[i];
            name = name.replace("//", "/");
            if (fs.statSync(name).isDirectory()) {
                iterateFolders(name, files_, relative_);
            } else if (isJsonFile(name) && name.includes('/containers/')) {
                relative_.push(name);
                files_.push(files[i]);
            }
        }
        return [files_, relative_];
    }

    function isJsonFile(fileName) {
        return fileName && ~fileName.indexOf(".json");
    }

    function writeFile(fullName, content, dontForce) {
        if (!dontForce || !fs.existsSync(fullName)) {
            printGruntLog("Building " + fullName + "...", SUCCESS);
            fs.writeFileSync(fullName, content);
        }
    }

    function getSubModule(containerPath) {
        let submodule = "";
        if (containerPath.includes("/modules/")) {
            let pathArray = containerPath.split("/");
            let modulesIndex = pathArray.indexOf("modules");
            submodule = "_" + pathArray[modulesIndex + 1];
        }
        return submodule;
    }

    function buildFile(buildPath, name, filter, containerPath, moduleName) {
        var fileName = name.split('.')[0];
        var submodule = getSubModule(containerPath);
        var fullName = buildPath + '/' + fileName + ".spec.js";
        var windowName = '#/' + moduleName + submodule + "#" + fileName;

        fullName = path.normalize(fullName);
        writeFile(fullName, buildContent(windowName, filter, buildPath));
        writeFile(fullName.replace(BUILDED_PATH, CUSTOM), buildCustomContent(windowName), dontForce);
    }

    function createBuildFolder(name) {
        try {
            shell.mkdir('-p', name);
        } catch (err) {
            printGruntLog(err, ERROR_TYPE);
        }
    }

    var buildFilesPathMode = function (dir, buildPath, moduleName) {
        var [containerNames, relativePath] = iterateFolders(dir);
        containerNames.forEach(function (container, index) {
            var containerPath = relativePath[index];
            buildFile(buildPath + BUILDED_PATH, container, null, containerPath, moduleName);
        });
    };

    grunt.registerTask("zhTestBuilder", "Generate tests for json containers", function () {
        var options = this.options({
            jsonPath: JSON_PATH,
            srcPath: SRC_PATH,
            moduleName: MODULE_NAME,
            buildPath: BUILD_PATH
        });

        validateOptions(options);

        var buildPath = options.buildPath;

        createBuildFolder(buildPath + BUILDED_PATH);
        createBuildFolder(buildPath + CUSTOM);

        buildFilesPathMode(options.srcPath, buildPath, options.moduleName);
    });
};
